@@ -18,8 +18,8 @@ Hopefully, you can just use the command line program:
            If $LINKTREE_BASE is set, <dest> may be omitted.
  
          linktree -p [-n] [-v] [-f LINKTREES] <dir>
-           -f <trees>    write a list of all directories linked into <dir>
            -p            prune bad links under <dir> (or $LINKTREE_BASE)
+           -f <trees>    sends the results to <trees>
  
            -n            no operation mode
            -v            verbose (print all commands)
@@ -37,7 +37,7 @@ use File::Recurse;
 use Cwd;
 use Carp;
 
-$VERSION = '1.03';
+$VERSION = '1.05';
 
 =head2 symlink_tree($opt)
 
@@ -64,17 +64,24 @@ Optional parameters are:
 sub symlink_tree {
     my ($o) = @_;
 
-    $o->{nop} ||= 0;
     $o->{'cwd'} ||= getcwd;
+    chdir($o->{'cwd'}) or die "chdir $o->{'cwd'}: $!";
+
+    $o->{nop} ||= 0;
     $o->{dec_mode} ||= oct($o->{mode} || '777');
 
     # get args
     my ($Src,$Dest) = ($o->{src}, $o->{dest});
     die "Directory $Src does not exist" if !-d $Src;
-    if (!-d $Dest) { mkdir($Dest, $o->{dec_mode}) or die "mkdir $Dest: $!"; }
-
+    if (!-d $Dest) {
+      my $d;
+      for my $d1 (split(m'/+', $Dest)) {
+	$d .= "$d1/";
+	if (!-d $d) { mkdir($d, $o->{dec_mode}) or die "mkdir $Dest: $!"; }
+      }
+    }
+    
     # get full pathnames
-    chdir($o->{'cwd'}) or die "chdir $o->{'cwd'}: $!";
     chdir($Dest) or die "chdir $Dest: $!";
     $Dest=getcwd;
 
@@ -100,7 +107,12 @@ sub symlink_tree {
 		    print "rm -f $d\n" if $o->{verbose};
 		    if (!$o->{nop}) { (unlink($d)==1) or die "unlink $d: $!";}
 		} elsif (-e $d) {
-		    die "$d exists and is not a symlink";
+		    if ($o->{force}) {
+			if (!$o->{nop}) { (unlink($d)==1) or die "unlink $d: $!";}
+		    } else {
+			warn "$d exists and is not a symlink\n";
+			return 0;
+		    }
 		}
 		if (-l $s) {
 		    my $link = readlink($s) or die "readlink($s): $!";
@@ -152,47 +164,63 @@ Optional parameters are:
 =cut
 
 sub prune_tree {
-    my ($o) = @_;
-
-    $o->{trees} ||= "$o->{dir}/LINKTREES";
-
-    my $fh = new IO::File;
-    $fh->open(">$o->{trees}") or die "open >$o->{trees}: $!";
-    my %trees;
-
-    chdir $o->{dir} or die "chdir $o->{dir}: $!";
-
-    print "prune_tree $o->{dir}\n" if $o->{verbose};
-
-    recurse(sub {
-	my $f = substr($_, 2);
-	if (-l $f) {
-	    if (!-e $f) {
+  my ($o) = @_;
+  
+  $o->{trees} ||= "$o->{dir}/LINKTREES";
+  
+  my $fh = new IO::File;
+  $fh->open(">$o->{trees}") or die "open >$o->{trees}: $!";
+  my %trees;
+  
+  chdir $o->{dir} or die "chdir $o->{dir}: $!";
+  
+  print "prune_tree $o->{dir}\n" if $o->{verbose};
+  
+  recurse(sub {
+	    my $f = substr($_, 2);
+	    if (-l $f) {
+	      if (!-e $f) {
 		print "rm -f $f\n" if $o->{verbose};
 		if (!$o->{nop}) { (unlink($f)==1) or die "unlink $f: $!"; }
-	    } else {
+	      } else {
 		my $path = readlink($f);
-		my $plen = length($path) - length($f);
+		my $plen = length($path) - length($f) ;
 		if ($plen > 0 and substr($path, $plen) eq $f) {
-		    $trees{ substr($path, 0, $plen) }=1;
+		  -- $plen;
+		  my $base = substr($path, 0, $plen);
+		  my $sub = substr($path, $plen);
+		  $sub =~ s,^(.*)/.*?$,$1,;
+		  $trees{ $base } = $sub if !exists $trees{ $base };
+#		  warn join(' ', $base, $trees{ $base }, $sub)."\n";
+		  while (length $trees{ $base }) {
+		    last if $trees{ $base } eq $sub;
+		    $sub =~ s,^(.*)/.*?$,$1,;
+		    $trees{ $base } =~ s,^(.*)/.*?$,$1,;
+		  }
 		}
-	    }
-	} elsif (-f $f) {
-	    if ("$o->{dir}/$f" ne $o->{trees}) {
+	      }
+	    } elsif (-f $f) {
+	      if ("$o->{dir}/$f" ne $o->{trees}) {
 		warn "Not a symlink: $f\n" if !$o->{silent};
-	    }
-	} elsif (-d $f) {
-	    if (!$o->{nop}) {
+	      }
+	    } elsif (-d $f) {
+	      if (!$o->{nop}) {
 		if (rmdir($f)) {
-		    print "rmdir $f\n" if $o->{verbose};
-		    return -1;
+		  print "rmdir $f\n" if $o->{verbose};
+		  return -1;
 		}
+	      }
 	    }
-	}
-	0;
-    }, '.');
-
-    map { print $fh "$_\n"; } sort keys %trees;
+	    0;
+	  }, '.');
+  
+  if (!$o->{nop}) {
+    for my $k (sort keys %trees) {
+      my $v = $trees{$k};
+#      warn "$k $v";
+      $fh->print($k.($v? " $v":'')."\n");
+    }
+  }
 }
 
 1;
@@ -200,7 +228,7 @@ __END__;
 
 =head1 BUGS
 
-Needs higher complexity regression tests.
+Needs more complex regression tests.
 
 =head1 AUTHOR
 
